# Copyright 2017 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import textwrap
import warnings

__all__ = ['warn', 'AliasUsageWarning']


class _BaseWarning(Warning):
    details: str


class AliasUsageWarning(_BaseWarning):
    """Use of historical service-type aliases is discouraged."""

    details = """
    Requested service_type {given} is an old alias. Please update your
    code to reference the official service_type {official}.
    """


class UnofficialUsageWarning(_BaseWarning):
    """Use of unofficial service-types is discouraged."""

    details = """
    Requested service_type {given} is not a known official OpenStack project.
    """


def warn(warning: type[_BaseWarning], **kwargs: str | None) -> None:
    """Emit a warning that has builtin message text."""
    message = textwrap.fill(textwrap.dedent(warning.details.format(**kwargs)))
    warnings.warn(message, category=warning)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_data
------------

Tests for `os_service_types.data` module.

"""

import json

from os_service_types import data
from os_service_types.tests import base


class TestData(base.TestCase, base.TemporaryFileMixin):
    def setUp(self):
        super().setUp()

    def test_load(self):
        json_data = {'some_key': 'some_value'}
        filename = self.create_json(json_data)
        actual_data = data.read_data(filename)
        self.assertEqual(json_data, actual_data)

    def test_load_service_types(self):
        json_data = data.read_data('service-types.json')
        for key in [
            "all_types_by_service_type",
            "forward",
            "primary_service_by_project",
            "reverse",
        ]:
            self.assertIn(key, json_data)

    def test_load_non_existing(self):
        self.assertRaises(
            FileNotFoundError, data.read_data, '/non-existing-file'
        )

    def create_json(self, json_data):
        fd, name = self.create_temp_file(suffix='.json')
        with fd:
            json.dump(json_data, fd)
        return name



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_singleton
------------

Tests for `get_service_types` singleton factory function.
"""

import os_service_types
from os_service_types.tests import base


class TestSingleton(base.TestCase):
    def setUp(self):
        super().setUp()
        # Make an object with no network access
        self.service_types = os_service_types.get_service_types()

    def test_singleton_same(self):
        service_types = os_service_types.get_service_types()
        self.assertTrue(service_types is self.service_types)

    def test_singleton_different(self):
        service_types = os_service_types.ServiceTypes()
        self.assertFalse(service_types is self.service_types)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import TypedDict

from typing_extensions import NotRequired, Required


class Service(TypedDict, total=False):
    aliases: NotRequired[list[str]]
    api_reference: Required[str]
    project: Required[str]
    service_type: Required[str]


class ServiceTypes(TypedDict):
    all_types_by_service_type: Required[dict[str, list[str]]]
    forward: Required[dict[str, list[str]]]
    primary_service_by_project: Required[dict[str, Service]]
    reverse: Required[dict[str, str]]
    service_types_by_project: Required[dict[str, list[str]]]
    services: Required[list[Service]]
    sha: Required[str]
    version: Required[str]



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_remote
-----------

Tests for `ServiceTypes` class remote data.

oslotest sets up a TempHomeDir for us, so there should be no ~/.cache files
available in these tests.
"""

from requests_mock.contrib import fixture as rm_fixture
from testscenarios import load_tests_apply_scenarios as load_tests  # noqa

import keystoneauth1.session

import os_service_types
import os_service_types.service_types
from os_service_types.tests import base


class TestRemote(base._BaseServiceDataTest):
    def setUp(self):
        super().setUp()
        # Set up a requests_mock fixture for all HTTP traffic
        adapter = self.useFixture(rm_fixture.Fixture())
        adapter.register_uri(
            'GET',
            os_service_types.service_types.SERVICE_TYPES_URL,
            json=self.remote_content,
            headers={'etag': self.getUniqueString('etag')},
        )
        # use keystoneauth1 to get a Sessiom with no auth information
        self.session = keystoneauth1.session.Session()
        # Make an object that fetches from the network
        self.service_types = os_service_types.ServiceTypes(
            session=self.session
        )
        self.assertEqual(1, len(adapter.request_history))

    def test_remote_version(self):
        self.assertEqual(self.remote_version, self.service_types.version)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_misc
---------

Miscellaneous tests
"""

from os_service_types import service_types
from os_service_types.tests import base


class TestMisc(base.TestCase):
    def test_normalize(self):
        self.assertEqual('foo-bar', service_types._normalize_type('foo_bar'))

    def test_normalize_none(self):
        self.assertIsNone(service_types._normalize_type(None))



# Copyright 2017 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import os

from os_service_types import types

__all__ = ['read_data']

DATA_DIR = os.path.dirname(__file__)


def read_data(filename: str) -> types.ServiceTypes:
    """Return data that is shipped inside the Python package."""

    filepath = os.path.join(DATA_DIR, filename)
    with open(filepath) as fd:
        data: types.ServiceTypes = json.load(fd)

    return data



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

__all__ = ['__version__', 'ServiceTypes']

from typing import TYPE_CHECKING, Optional

import pbr.version

from os_service_types.service_types import ServiceTypes

if TYPE_CHECKING:
    from requests import sessions

__version__ = pbr.version.VersionInfo('os-service-types').version_string()
_service_type_manager = None


def get_service_types(
    session: Optional['sessions.Session'] = None,
    only_remote: bool = False,
    warn: bool = False,
) -> ServiceTypes:
    """Return singleton instance of the ServiceTypes object.

    Parameters are all passed through to the
    :class:`~os_service_types.service_types.ServiceTypes` constructor.

    .. note::

        Only one singleton is kept, so if instances with different parameter
        values are desired, directly calling the constructor is necessary.

    :returns: Singleton instance of
        :class:`~os_service_types.service_types.ServiceTypes`
    """
    global _service_type_manager
    if not _service_type_manager:
        _service_type_manager = ServiceTypes(
            session, only_remote=only_remote, warn=warn
        )
    return _service_type_manager
